==> project-tasks-scheduling-engine-0.0.3.tgz: package/src/scheduleTasks.js <==
import { max, subBusinessDays, addBusinessDays, isEqual } from "date-fns";
import { makeGraphFromTasks, makeReverseGraph, dfs } from "./graph.utils";
import { shiftToFirstNextBusinessDay } from "./date.utils";
export const scheduleTasks = (tasks, today) => {
    const graph = makeGraphFromTasks(tasks);
    const tasksById = tasks.reduce((map, task) => {
        map[task.id] = task;
        return map;
    }, {});
    // @TODO: 0. Detect cycles, if present throw error
    // 1. Make reverse graph, to detect sinks and sources
    const reverseGraph = makeReverseGraph(graph);
    // 2. If node is source, t.start = max(today, t.start)
    dfs(graph, id => {
        const t = tasksById[id];
        const isSource = reverseGraph.get(id)?.size === 0;
        const isSink = graph.get(id)?.size === 0;
        const isDisconnected = isSource && isSink;
        if (isSource || isDisconnected) {
            updateStartDate(t, today ?? new Date());
        }
        else {
            const prerequesionsEndDates = Array.from(reverseGraph.get(id) ?? []).map(id => tasksById[id].end);
            updateStartDate(t, addBusinessDays(max(prerequesionsEndDates), 1));
        }
    });
    return tasks;
};
export const updateStartDate = (task, startDate) => {
    const correctedStartDate = shiftToFirstNextBusinessDay(startDate);
    const daysSpent = Math.floor(task.duration * task.progress);
    const newStartDate = subBusinessDays(correctedStartDate, daysSpent);
    if (isEqual(task.start, newStartDate)) {
        return;
    }
    task.start = subBusinessDays(correctedStartDate, daysSpent);
    // -1 because every task is minimum 1 day long,
    // say it starts and ends on same date, so it has 1 day duration
    task.end = addBusinessDays(task.start, task.duration - 1);
};

==> project-tasks-scheduling-engine-0.1.0.tgz: package/src/scheduleTasks.js <==
import { max, subBusinessDays, addBusinessDays, isEqual } from "date-fns";
import { makeGraphFromTasks, makeReverseGraph, dfsWithRepetitions, } from "./graph.utils";
import { shiftToFirstNextBusinessDay } from "./date.utils";
export const scheduleTasks = (tasks, today) => {
    const graph = makeGraphFromTasks(tasks);
    const tasksById = tasks.reduce((map, task) => {
        map[task.id] = task;
        return map;
    }, {});
    // @TODO: 0. Detect cycles, if present throw error
    // 1. Make reverse graph, to detect sinks and sources
    const reverseGraph = makeReverseGraph(graph);
    // 2. If node is source, t.start = max(today, t.start)
    for (const [id] of dfsWithRepetitions(graph)) {
        const t = tasksById[id];
        const isSource = reverseGraph.get(id)?.size === 0;
        const isSink = graph.get(id)?.size === 0;
        const isDisconnected = isSource && isSink;
        if (isSource || isDisconnected) {
            updateStartDate(t, today ?? new Date());
        }
        else {
            const prerequesionsEndDates = Array.from(reverseGraph.get(id) ?? []).map((id) => tasksById[id].end);
            updateStartDate(t, addBusinessDays(max(prerequesionsEndDates), 1));
        }
    }
    return tasks;
};
export const updateStartDate = (task, startDate) => {
    const correctedStartDate = shiftToFirstNextBusinessDay(startDate);
    const daysSpent = Math.floor(task.duration * task.progress);
    const newStartDate = subBusinessDays(correctedStartDate, daysSpent);
    if (isEqual(task.start, newStartDate)) {
        return;
    }
    task.start = subBusinessDays(correctedStartDate, daysSpent);
    // -1 because every task is minimum 1 day long,
    // say it starts and ends on same date, so it has 1 day duration
    task.end = addBusinessDays(task.start, task.duration - 1);
};
